import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.Timeout;

import java.lang.reflect.Field;

import static org.hamcrest.CoreMatchers.is;

public class Aufgabe1Test {

    @Rule
    public Timeout globalTimeout = Timeout.millis(1000);

    @Test
    public void testEquals() throws Exception {
        Position p = new Position(1,2);
        for (int x = 0; x < 5; x++) {
            for (int y = 0; y < 5; y++) {
                Assert.assertTrue((x == 1 && y == 2) == new Position(x,y).equals(p));
            }
        }

    }

    @Test
    public void testMoveVerticalAndEquals() throws Exception {
        Position p = new Position(1,2);
        p.moveVertical(5);
        Assert.assertTrue(p.equals(new Position(1,7)));

    }

    @Test
    public void testIsNeighbour() throws Exception {
        Position p = new Position(1,2);;
        Assert.assertTrue(p.isNeighbour(new Position(1,3)));
        Assert.assertTrue(p.isNeighbour(new Position(1,1)));
        Assert.assertTrue(p.isNeighbour(new Position(2,3)));
        Assert.assertTrue(p.isNeighbour(new Position(0,1)));
        Assert.assertTrue(p.isNeighbour(new Position(0,2)));
        Assert.assertTrue(p.isNeighbour(new Position(2,2)));
        Assert.assertTrue(p.isNeighbour(new Position(0,3)));
        Assert.assertTrue(p.isNeighbour(new Position(1,2)));
        Assert.assertTrue(!p.isNeighbour(new Position(3,2)));
        Assert.assertTrue(!p.isNeighbour(new Position(3,3)));
        Assert.assertTrue(!p.isNeighbour(new Position(1,0)));
        Assert.assertTrue(!p.isNeighbour(new Position(1,4)));
        Assert.assertTrue(!p.isNeighbour(new Position(2,4)));
        Assert.assertTrue(!p.isNeighbour(new Position(2,0)));

    }

    @Test
    public void testSetBomb() throws Exception {
        Minefield f = new Minefield(5, 7);
        Assert.assertTrue(f.setBomb(3, 2));
        Assert.assertTrue(!f.setBomb(5, 3));
        Assert.assertTrue(f.setBomb(4, 5));
        Assert.assertTrue(!f.setBomb(4, 9));
        Assert.assertTrue(!f.setBomb(3, 2));
    }

    @Test
    public void testBombNeighbours() throws Exception {
        Minefield f = new Minefield(10, 10);
        f.setBomb(3, 2);
        f.setBomb(5, 3);
        f.setBomb(4, 5);
        f.setBomb(6, 5);
        f.setBomb(4, 9);
        Assert.assertThat(f.bombNeighbours(4, 4),is(2));
        Assert.assertThat(f.bombNeighbours(5, 4),is(3));
        Assert.assertThat(f.bombNeighbours(1, 1),is(0));
    }

    @Test
    public void testToString() throws Exception {
        Minefield f = new Minefield(10, 13);
        f.setBomb(3, 2);
        f.setBomb(5, 3);
        f.setBomb(4, 5);
        f.setBomb(6, 5);
        f.setBomb(9, 9);
        f.setBomb(4, 9);
        String result = "0000000000\n" +
                "0011100000\n" +
                "001*211000\n" +
                "00112*1000\n" +
                "0001232100\n" +
                "0001*2*100\n" +
                "0001121100\n" +
                "0000000000\n" +
                "0001110011\n" +
                "0001*1001*\n" +
                "0001110011\n" +
                "0000000000\n" +
                "0000000000\n" +
                "10 13";

        Assert.assertThat(f.toString(), is(result));
    }

    @Test
    public void testTrimSetBomb() throws Exception {
        Minefield f = new Minefield(10, 13);
        f.setBomb(3, 2);
        f.setBomb(5, 3);
        f.setBomb(4, 5);
        f.setBomb(6, 5);
        f.setBomb(9, 9);
        f.setBomb(4, 9);
        f.trim();
        Assert.assertTrue(f.isBomb(3,0));
        Assert.assertTrue(f.isBomb(5,1));
        Assert.assertTrue(f.isBomb(4,3));
        Assert.assertTrue(f.isBomb(6,3));
        Assert.assertTrue(f.isBomb(9,7));
        Assert.assertTrue(f.isBomb(4,7));
        Assert.assertTrue(!f.isBomb(1,1));
        Assert.assertTrue(!f.isBomb(2,2));
        Assert.assertTrue(!f.isBomb(2,3));
        Assert.assertTrue(!f.isBomb(3,5));
        Assert.assertTrue(!f.isBomb(3,2));

    }

    @Test
    public void testTrimToString() throws Exception {
        Minefield f = new Minefield(10, 13);
        f.setBomb(3, 2);
        f.setBomb(5, 3);
        f.setBomb(4, 5);
        f.setBomb(6, 5);
        f.setBomb(9, 9);
        f.setBomb(4, 9);
        f.trim();
        String result =
                "001*211000\n" +
                        "00112*1000\n" +
                        "0001232100\n" +
                        "0001*2*100\n" +
                        "0001121100\n" +
                        "0000000000\n" +
                        "0001110011\n" +
                        "0001*1001*\n" +
                        "10 8";
        Assert.assertThat(f.toString(), is(result));
    }

    @Test
    public void testTrimSize() throws Exception {
        Minefield f = new Minefield(10, 13);
        f.setBomb(3, 2);
        f.setBomb(5, 3);
        f.setBomb(4, 5);
        f.setBomb(6, 5);
        f.setBomb(9, 9);
        f.setBomb(4, 9);
        f.trim();
        Field field = Minefield.class.getDeclaredField("width");
        field.setAccessible(true);
        int width = (int) field.get(f);

        field = Minefield.class.getDeclaredField("height");
        field.setAccessible(true);
        int height = (int) field.get(f);

        Assert.assertTrue(width == 10 && height == 8);
    }

    @Test
    public void testCopyAndTrimToString() throws Exception {
        Minefield f = new Minefield(10, 13);
        f.setBomb(3, 2);
        f.setBomb(5, 3);
        f.setBomb(4, 5);
        f.setBomb(6, 5);
        f.setBomb(4, 9);
        f.setBomb(9, 9);
        Minefield g = f.copy();
        g.trim();
        String resultG =
                "001*211000\n" +
                        "00112*1000\n" +
                        "0001232100\n" +
                        "0001*2*100\n" +
                        "0001121100\n" +
                        "0000000000\n" +
                        "0001110011\n" +
                        "0001*1001*\n" +
                        "10 8";
        String resultF = "0000000000\n" +
                "0011100000\n" +
                "001*211000\n" +
                "00112*1000\n" +
                "0001232100\n" +
                "0001*2*100\n" +
                "0001121100\n" +
                "0000000000\n" +
                "0001110011\n" +
                "0001*1001*\n" +
                "0001110011\n" +
                "0000000000\n" +
                "0000000000\n" +
                "10 13";
        Assert.assertThat(g.toString().trim(),is(resultG));
        Assert.assertThat(f.toString().trim(),is(resultF));

    }

}